/**********************************************************
 * UTILITY FUNCTIONS
 **********************************************************/
const searchAndFindTokenContainer = async (selector) => {
  const interval = 600;
  const endTime = Date.now() + 12000;
  while (Date.now() < endTime) {
    const container = document.querySelector(selector);
    if (container) return container;
    await new Promise((r) => setTimeout(r, interval));
  }
  return null;
};

const searchAndFindChartsContainer = async (selector) => {
  const interval = 1000;
  const endTime = Date.now() + 20000;
  while (Date.now() < endTime) {
    const chartsContainer = document.querySelector(selector);
    if (chartsContainer) return chartsContainer;
    await new Promise((r) => setTimeout(r, interval));
  }
  return null;
};

const searchAndFindBuyAndSellContainer = async (selector) => {
  const interval = 600;
  const endTime = Date.now() + 12000;
  while (Date.now() < endTime) {
    const container = document.querySelector(selector);
    if (container) return container;
    await new Promise((r) => setTimeout(r, interval));
  }
  return null;
};

/**********************************************************
 * TOKEN PAGE INJECTION FUNCTIONS
 **********************************************************/
const injectNovaSnipeButton = () => {
  try {
    let novaAuthToken, elementsClassname, srcn;
    chrome.storage.local.get(
      ["nova_auth_token", "elements_classname", "srcn"],
      (result) => {
        elementsClassname = result.elements_classname;
        novaAuthToken = result.nova_auth_token;

        if (!result.srcn) {
          const createdClassname = createRandomClassname();
          chrome.storage.local.set({ srcn: createdClassname });
          srcn = createdClassname;
        } else {
          srcn = result.srcn;
        }

        const migrationParent = document.querySelector(
          elementsClassname?.placement_selectors?.axiom?.token?.snipe?.parent
        );
        const migrationElements = Array.from(
          document.querySelectorAll(
            elementsClassname?.placement_selectors?.axiom?.token?.snipe
              ?.migrating
          )
        );

        migrationElements.forEach((mE) => {
          // Remove any previous sniping button
          const previousSnipingButton = document.querySelector(
            `button.${CSS.escape(srcn)}`
          );
          if (previousSnipingButton) previousSnipingButton.remove();

          if (mE) {
            const migratingParagraph = document.querySelector(
              elementsClassname?.placement_selectors?.axiom?.token?.snipe
                ?.paragraph
            );
            if (!migratingParagraph) return;

            const customButton = document.createElement("button");
            const buttonImg = document.createElement("img");
            buttonImg.src = elementsClassname?.pp;
            buttonImg.alt = "";
            buttonImg.style.aspectRatio = "1/1";
            buttonImg.style.height = "15px";
            buttonImg.style.width = "15px";

            const buttonText = document.createElement("span");
            buttonText.textContent = "Snipe";

            customButton.appendChild(buttonText);
            customButton.type = "button";
            customButton.classList.add(srcn);
            customButton.style.marginTop = "6px";
            customButton.style.width = "87.5%";
            customButton.style.padding = "4px 8px";
            customButton.style.position = "absolute";
            customButton.style.zIndex = "1000";
            customButton.style.bottom = "0";
            customButton.style.borderRadius = "8px";
            customButton.style.border = elementsClassname?.br?.b;

            customButton.onclick = async function () {
              const url = new URL(window.location.href);
              const tokenMintAddress = url.pathname.split("/meme/")[1];

              chrome.storage.local.get("default_buy_amount", async (r) => {
                const defaultBuyAmount = r.default_buy_amount || 0.01;
                customButton.disabled = true;
                customButton.querySelector("span").textContent =
                  "Processing...";

                const tokenInfo = await fetchTokenInfo(tokenMintAddress);
                const result = await transactToken(
                  tokenInfo.tokenAddress,
                  "snipe",
                  defaultBuyAmount,
                  novaAuthToken,
                  {}
                );

                if (result) {
                  customButton.querySelector("span").textContent = "Success!";
                  setTimeout(() => {
                    customButton.querySelector("span").textContent = "Snipe";
                    customButton.disabled = false;
                  }, 700);
                } else {
                  customButton.querySelector("span").textContent = "Failed!";
                  setTimeout(() => {
                    customButton.querySelector("span").textContent = "Snipe";
                    customButton.disabled = false;
                  }, 700);
                }
              });
            };

            migrationParent.appendChild(customButton);
          }
        });
      }
    );
  } catch (error) {
    console.error(error);
  }
};

const injectNovaContainer = () => {
  try {
    let novaAuthToken;
    let elementsClassname;
    let buyButtonsList;
    let sellButtonsList;
    chrome.storage.local
      .get([
        "nova_auth_token",
        "is_buy_axiom",
        "custom_buy_value_list",
        "custom_sell_value_list",
        "elements_classname",
      ])
      .then((result) => {
        elementsClassname = result.elements_classname;
        novaAuthToken = result.nova_auth_token;

        if (
          Array.isArray(result.custom_buy_value_list) &&
          result.custom_buy_value_list.length > 0
        ) {
          buyButtonsList = result.custom_buy_value_list;
        } else {
          buyButtonsList = [0.5, 1, 2, 5, 10];
        }
        if (
          Array.isArray(result.custom_sell_value_list) &&
          result.custom_sell_value_list.length > 0
        ) {
          sellButtonsList = result.custom_sell_value_list;
        } else {
          sellButtonsList = [10, 25, 50, 100];
        }

        const url = new URL(window.location.href);
        let tokenMintAddress = url.pathname.split("/meme/")[1];
        if (!tokenMintAddress) return;

        const previousBuyAndSellButtonsContainer = document.querySelector(
          `div.${CSS.escape(elementsClassname?.bsc)}`
        );
        if (previousBuyAndSellButtonsContainer) {
          previousBuyAndSellButtonsContainer.remove();
        }

        const chartsContainer = document.querySelector(
          elementsClassname?.placement_selectors?.axiom?.token?.container
        ).parentElement.parentElement.parentElement.parentElement;

        if (!chartsContainer) return;

        let isBuy = result.is_buy_axiom;

        const generatedNovaBuyAndSellButtonsContainerClassName =
          elementsClassname?.bsc;
        const buyAndSellButtonsContainer = document.createElement("div");
        buyAndSellButtonsContainer.classList.add(
          generatedNovaBuyAndSellButtonsContainerClassName
        );
        buyAndSellButtonsContainer.style.width = "100%";
        buyAndSellButtonsContainer.style.marginTop = "8px";
        buyAndSellButtonsContainer.style.paddingLeft = "12px";
        buyAndSellButtonsContainer.style.paddingRight = "12px";
        buyAndSellButtonsContainer.style.paddingBottom = "10px";
        buyAndSellButtonsContainer.style.display = "flex";
        buyAndSellButtonsContainer.style.flexDirection = "column";
        buyAndSellButtonsContainer.style.justifyContent = "start";
        buyAndSellButtonsContainer.style.alignItems = "start";
        buyAndSellButtonsContainer.style.gap = "12px";

        const toggleBuyOrSellButton = document.createElement("button");
        toggleBuyOrSellButton.type = "button";
        toggleBuyOrSellButton.style.padding = "8px 14px";
        toggleBuyOrSellButton.style.background = "#44103f";
        toggleBuyOrSellButton.style.border = elementsClassname?.br?.tg;
        toggleBuyOrSellButton.style.borderRadius = "8px";
        toggleBuyOrSellButton.textContent = isBuy
          ? elementsClassname?.container?.switch_btn_content?.sell
          : elementsClassname?.container?.switch_btn_content?.buy;
        toggleBuyOrSellButton.style.textWrap = "nowrap";
        toggleBuyOrSellButton.style.fontSize = "14px";
        toggleBuyOrSellButton.style.fontWeight = "600";
        toggleBuyOrSellButton.style.transition = ".2 ease-in-out";

        toggleBuyOrSellButton.addEventListener("mouseenter", () => {
          toggleBuyOrSellButton.style.background = "#5a1353";
        });
        toggleBuyOrSellButton.addEventListener("mouseleave", () => {
          toggleBuyOrSellButton.style.background = "#44103f";
        });

        toggleBuyOrSellButton.addEventListener("click", () => {
          if (buyContainer.style.display === "flex") {
            fieldContainer.replaceChild(sellOptionsContainer, buyLabelOption);
            isBuy = false;
            chrome.storage.local.set(
              {
                is_buy_axiom: false,
              },
              () => {}
            );
            buttonText.textContent = "Sell";
            toggleBuyOrSellButton.textContent =
              elementsClassname?.container?.switch_btn_content?.buy;
            input.setAttribute(
              "placeholder",
              activeSellOption === "%"
                ? elementsClassname?.container?.placeholders?.sell?.percentage
                : elementsClassname?.container?.placeholders?.sell?.amount
            );
            input.value = "";
            buyContainer.style.display = "none";
            sellContainer.style.display = "flex";
          } else {
            fieldContainer.replaceChild(buyLabelOption, sellOptionsContainer);
            isBuy = true;
            chrome.storage.local.set(
              {
                is_buy_axiom: true,
              },
              () => {}
            );
            buttonText.textContent = "Buy";
            toggleBuyOrSellButton.textContent =
              elementsClassname?.container?.switch_btn_content?.sell;
            input.setAttribute(
              "placeholder",
              elementsClassname?.container?.placeholders?.buy?.amount
            );
            input.value = "";
            buyContainer.style.display = "flex";
            sellContainer.style.display = "none";
          }
        });

        const buyContainer = document.createElement("div");
        buyContainer.style.display = isBuy ? "flex" : "none";
        buyContainer.style.justifyContent = "start";
        buyContainer.style.alignItems = "center";
        buyContainer.style.columnGap = "8px";
        buyContainer.style.rowGap = "8px";
        const sellContainer = document.createElement("div");
        sellContainer.style.display = isBuy ? "none" : "flex";
        sellContainer.style.justifyContent = "start";
        sellContainer.style.alignItems = "center";
        sellContainer.style.columnGap = "8px";
        sellContainer.style.rowGap = "8px";

        const customBuyAndSellContainer = document.createElement("div");
        customBuyAndSellContainer.style.display = "flex";
        customBuyAndSellContainer.style.justifyContent = "start";
        customBuyAndSellContainer.style.alignItems = "center";
        customBuyAndSellContainer.style.columnGap = "8px";
        customBuyAndSellContainer.style.rowGap = "8px";

        const fieldContainer = document.createElement("div");
        Object.assign(fieldContainer.style, {
          boxSizing: "border-box",
          display: "flex",
          width: "270px",
          justifyContent: "center",
          background: "#121212",
          height: "36px",
          border: "1px solid rgb(38 40 44)",
          borderRadius: "8px",
          overflow: "hidden",
        });

        const buyLabelOption = document.createElement("div");
        Object.assign(buyLabelOption.style, {
          display: "flex",
          whiteSpace: "nowrap",
          background: "inherit",
          border: "none",
          height: "100%",
          padding: "0 12px",
          justifyContent: "center",
          alignItems: "center",
          color: "white",
          fontWeight: "bold",
          fontSize: "12px",
          borderRight: "1px solid rgb(38 40 44)",
        });
        buyLabelOption.textContent = "SOL";

        const input = document.createElement("input");

        let activeSellOption = "";
        const sellOptions = ["%", "SOL"];
        const sellOptionsContainer = document.createElement("div");
        Object.assign(sellOptionsContainer.style, {
          display: "flex",
          alignItems: "center",
          borderRight: "1px solid rgb(38 40 44)",
        });
        const setActiveOption = (optionElement) => {
          Array.from(sellOptionsContainer.children).forEach((child) => {
            Object.assign(child.style, {
              color: "gray",
              fontWeight: "normal",
            });
          });
          Object.assign(optionElement.style, {
            color: "white",
            fontWeight: "bold",
          });
          activeSellOption = optionElement.textContent;

          input.placeholder =
            optionElement.textContent === "%"
              ? elementsClassname?.container?.placeholders?.sell?.percentage
              : elementsClassname?.container?.placeholders?.sell?.amount;
        };
        sellOptions.forEach((option, index) => {
          const sellLabelOption = document.createElement("button");
          Object.assign(sellLabelOption.style, {
            display: "flex",
            whiteSpace: "nowrap",
            background: "inherit",
            border: "none",
            height: "100%",
            padding: "0 12px",
            justifyContent: "center",
            alignItems: "center",
            color: "gray",
            fontSize: "12px",
            cursor: "pointer",
          });
          sellLabelOption.textContent = option;
          sellLabelOption.addEventListener("click", () => {
            input.value = "";
            setActiveOption(sellLabelOption);
          });
          sellOptionsContainer.appendChild(sellLabelOption);
          if (index < sellOptions.length - 1) {
            const separator = document.createElement("div");
            Object.assign(separator.style, {
              height: "50%",
              borderRight: "1px solid rgb(38 40 44)",
            });
            sellOptionsContainer.appendChild(separator);
          }
          if (index === 0) {
            setActiveOption(sellLabelOption);
          }
        });
        Object.assign(input.style, {
          width: "100%",
          height: "100%",
          borderRadius: "0.375rem",
          minWidth: "0px",
          outline: "transparent solid 2px",
          outlineOffset: "2px",
          position: "relative",
          appearance: "none",
          verticalAlign: "top",
          background: "inherit",
          textAlign: "left",
          fontSize: "12px",
          border: "none",
          outline: "none",
          color: "white",
          fontWeight: "500",
          padding: "0 12px",
        });
        input.type = "number";
        input.value = "";
        input.setAttribute("role", "spinbutton");
        input.setAttribute("autocomplete", "off");
        input.setAttribute("autocorrect", "off");
        input.setAttribute("name", elementsClassname?.customs?.bs);
        input.setAttribute(
          "placeholder",
          isBuy
            ? elementsClassname?.container?.placeholders?.buy?.amount
            : activeSellOption === "%"
              ? elementsClassname?.container?.placeholders?.sell?.percentage
              : elementsClassname?.container?.placeholders?.sell?.amount
        );
        input.setAttribute("type", "number");
        input.setAttribute("step", "2");
        input.style.maxWidth = "100%";
        input.addEventListener("input", (e) => {
          const value = e.target.value;

          if (isBuy) {
            const isValid = /^[1-9]\d*(\.\d+)?$|^0(\.\d+)?$/.test(value);

            if (!isValid) {
              e.target.value = value.slice(0, -1);
            }
          } else {
            const isValid =
              activeSellOption === "%"
                ? /^[1-9]$|^[1-9][0-9]$|^100$/.test(value)
                : /^[1-9]\d*(\.\d+)?$|^0(\.\d+)?$/.test(value);

            if (!isValid) {
              e.target.value = value.slice(0, -1);
            }
          }
        });

        if (isBuy) {
          fieldContainer.appendChild(buyLabelOption);
          fieldContainer.appendChild(input);
        } else {
          fieldContainer.appendChild(sellOptionsContainer);
          fieldContainer.appendChild(input);
        }

        const buyOrSellButton = document.createElement("button");
        const buttonText = document.createElement("span");
        const buttonImg = document.createElement("img");
        buttonImg.src = elementsClassname?.pp;
        buttonImg.alt = "";
        buttonImg.style.aspectRatio = "1/1";
        buttonImg.style.height = "15px";
        buttonImg.style.width = "15px";
        buttonImg.style.marginRight = "5px";
        buttonText.textContent = isBuy ? "Buy" : "Sell";
        buttonText.style.fontWeight = "600";
        buyOrSellButton.appendChild(buttonText);

        buyOrSellButton.type = "button";
        buyOrSellButton.style.boxSizing = "border-box";
        buyOrSellButton.style.width = "max-content";
        buyOrSellButton.style.display = "flex";
        buyOrSellButton.style.justifyContent = "center";
        buyOrSellButton.style.alignItems = "center";
        buyOrSellButton.style.gap = "4px";
        buyOrSellButton.style.background = "rgb(38 40 44)";
        buyOrSellButton.style.padding = "6px 12px";
        buyOrSellButton.style.marginRight = "6px";
        buyOrSellButton.style.borderRadius = "8px";
        buyOrSellButton.style.flexShrink = "0";
        buyOrSellButton.style.transition = ".2 ease-in-out";

        buyOrSellButton.addEventListener("mouseenter", () => {
          buyOrSellButton.style.background = "rgb(63 66 73)";
        });
        buyOrSellButton.addEventListener("mouseleave", () => {
          buyOrSellButton.style.background = "rgb(38 40 44)";
        });

        buyOrSellButton.addEventListener("click", async () => {
          const inputElement = document.querySelector(
            `[name=${elementsClassname?.customs?.bs}]`
          );
          const buyOrSellValue = parseFloat(inputElement.value);
          buyOrSellButton.disabled = true;
          buyOrSellButton.querySelector("span").textContent = "Processing...";

          try {
            const storageData = await new Promise((resolve, reject) => {
              chrome.storage.local.get("active_preset_values", (result) => {
                if (chrome.runtime.lastError) {
                  reject(chrome.runtime.lastError);
                } else {
                  resolve(result);
                }
              });
            });

            const tokenInfo = await fetchTokenInfo(tokenMintAddress);
            const result = await transactToken(
              tokenInfo.tokenAddress,
              isBuy ? "buy" : "sell",
              isBuy
                ? buyOrSellValue
                : activeSellOption === "SOL"
                  ? null
                  : buyOrSellValue,
              novaAuthToken,
              storageData?.active_preset_values,
              isBuy ? null : activeSellOption === "SOL" ? buyOrSellValue : null
            );

            if (result) {
              buyOrSellButton.querySelector("span").textContent = "Success!";
              setTimeout(() => {
                buyOrSellButton.querySelector("span").textContent = isBuy
                  ? "Buy"
                  : "Sell";
                buyOrSellButton.disabled = false;
              }, 700);
            } else {
              buyOrSellButton.querySelector("span").textContent = "Failed!";
              setTimeout(() => {
                buyOrSellButton.querySelector("span").textContent = isBuy
                  ? "Buy"
                  : "Sell";
                buyOrSellButton.disabled = false;
              }, 700);
            }
          } catch (error) {
            console.error("Error accessing Chrome storage:", error);
            buyOrSellButton.querySelector("span").textContent = "Error!";
            setTimeout(() => {
              buyOrSellButton.querySelector("span").textContent = isBuy
                ? "Buy"
                : "Sell";
              buyOrSellButton.disabled = false;
            }, 700);
          }
        });

        customBuyAndSellContainer.appendChild(fieldContainer);
        customBuyAndSellContainer.appendChild(buyOrSellButton);

        buyButtonsList.map((value) => {
          (function (buttonValue) {
            const buttonImg = document.createElement("img");
            buttonImg.src = elementsClassname?.pp;
            buttonImg.alt = "";
            buttonImg.style.aspectRatio = "1/1";
            buttonImg.style.height = "15px";
            buttonImg.style.width = "15px";
            buttonImg.style.marginRight = "5px";

            const buttonText = document.createElement("span");
            buttonText.textContent = `Buy ${buttonValue} SOL`;
            buttonText.style.fontWeight = "600";

            const customBuyButton = document.createElement("button");
            customBuyButton.appendChild(buttonText);
            customBuyButton.type = "button";
            customBuyButton.style.boxSizing = "border-box";
            customBuyButton.style.width = "max-content";
            customBuyButton.style.display = "flex";
            customBuyButton.style.justifyContent = "center";
            customBuyButton.style.alignItems = "center";
            customBuyButton.style.gap = "4px";
            customBuyButton.style.background = "rgb(38 40 44)";
            customBuyButton.style.padding = "6px 12px";
            customBuyButton.style.marginRight = "6px";
            customBuyButton.style.borderRadius = "8px";
            customBuyButton.style.transition = ".2 ease-in-out";
            customBuyButton.addEventListener("mouseenter", () => {
              customBuyButton.style.background = "rgb(63 66 73)";
            });
            customBuyButton.addEventListener("mouseleave", () => {
              customBuyButton.style.background = "rgb(38 40 44)";
            });

            customBuyButton.onclick = async function () {
              customBuyButton.disabled = true;
              customBuyButton.querySelector("span").textContent =
                "Processing...";

              try {
                const storageData = await new Promise((resolve, reject) => {
                  chrome.storage.local.get("active_preset_values", (result) => {
                    if (chrome.runtime.lastError) {
                      reject(chrome.runtime.lastError);
                    } else {
                      resolve(result);
                    }
                  });
                });

                const tokenInfo = await fetchTokenInfo(tokenMintAddress);
                const result = await transactToken(
                  tokenInfo.tokenAddress,
                  "buy",
                  buttonValue,
                  novaAuthToken,
                  storageData?.active_preset_values
                );

                if (result) {
                  customBuyButton.querySelector("span").textContent =
                    "Success!";
                  setTimeout(() => {
                    customBuyButton.disabled = false;
                    customBuyButton.querySelector("span").textContent =
                      `Buy ${buttonValue} SOL`;
                  }, 700);
                } else {
                  customBuyButton.querySelector("span").textContent = "Failed!";
                  setTimeout(() => {
                    customBuyButton.disabled = false;
                    customBuyButton.querySelector("span").textContent =
                      `Buy ${buttonValue} SOL`;
                  }, 700);
                }
              } catch (error) {
                console.error("Error accessing Chrome storage:", error);
                customBuyButton.querySelector("span").textContent = "Error!";
                setTimeout(() => {
                  customBuyButton.disabled = false;
                  customBuyButton.querySelector("span").textContent =
                    `Buy ${buttonValue} SOL`;
                }, 700);
              }
            };

            buyContainer.append(customBuyButton);
          })(value);
        });
        sellButtonsList.map((value) => {
          (function (buttonValue) {
            const buttonImg = document.createElement("img");
            buttonImg.src = elementsClassname?.pp;
            buttonImg.alt = "";
            buttonImg.style.aspectRatio = "1/1";
            buttonImg.style.height = "15px";
            buttonImg.style.width = "15px";
            buttonImg.style.marginRight = "5px";

            const buttonText = document.createElement("span");
            buttonText.textContent = `Sell ${buttonValue}%`;
            buttonText.style.fontWeight = "600";

            const customSellButton = document.createElement("button");
            customSellButton.appendChild(buttonText);
            customSellButton.type = "button";
            customSellButton.style.boxSizing = "border-box";
            customSellButton.style.width = "max-content";
            customSellButton.style.display = "flex";
            customSellButton.style.justifyContent = "center";
            customSellButton.style.alignItems = "center";
            customSellButton.style.gap = "4px";
            customSellButton.style.background = "rgb(38 40 44)";
            customSellButton.style.padding = "6px 12px";
            customSellButton.style.marginRight = "6px";
            customSellButton.style.borderRadius = "8px";
            customSellButton.style.transition = ".2 ease-in-out";
            customSellButton.addEventListener("mouseenter", () => {
              customSellButton.style.background = "rgb(63 66 73)";
            });
            customSellButton.addEventListener("mouseleave", () => {
              customSellButton.style.background = "rgb(38 40 44)";
            });

            customSellButton.onclick = async function () {
              customSellButton.disabled = true;
              customSellButton.querySelector("span").textContent =
                "Processing...";

              try {
                const storageData = await new Promise((resolve, reject) => {
                  chrome.storage.local.get("active_preset_values", (result) => {
                    if (chrome.runtime.lastError) {
                      reject(chrome.runtime.lastError);
                    } else {
                      resolve(result);
                    }
                  });
                });

                const tokenInfo = await fetchTokenInfo(tokenMintAddress);
                const result = await transactToken(
                  tokenInfo.tokenAddress,
                  "sell",
                  buttonValue,
                  novaAuthToken,
                  storageData?.active_preset_values
                );

                if (result) {
                  customSellButton.querySelector("span").textContent =
                    "Success!";
                  setTimeout(() => {
                    customSellButton.disabled = false;
                    customSellButton.querySelector("span").textContent =
                      `Sell ${buttonValue}%`;
                  }, 700);
                } else {
                  customSellButton.querySelector("span").textContent =
                    "Failed!";
                  setTimeout(() => {
                    customSellButton.disabled = false;
                    customSellButton.querySelector("span").textContent =
                      `Sell ${buttonValue}%`;
                  }, 700);
                }
              } catch (error) {
                console.error("Error accessing Chrome storage:", error);
                customSellButton.querySelector("span").textContent = "Error!";
                setTimeout(() => {
                  customSellButton.disabled = false;
                  customSellButton.querySelector("span").textContent =
                    `Sell ${buttonValue}%`;
                }, 700);
              }
            };

            sellContainer.append(customSellButton);
          })(value);
        });

        const buyAndSellWrapper = document.createElement("div");
        buyAndSellWrapper.style.display = "flex";
        buyAndSellWrapper.style.gap = "8px";
        const customBuyAndSellWithToggleWrapper = document.createElement("div");
        customBuyAndSellWithToggleWrapper.style.display = "flex";
        customBuyAndSellWithToggleWrapper.style.gap = "8px";

        buyAndSellWrapper.append(buyContainer);
        buyAndSellWrapper.append(sellContainer);
        buyAndSellButtonsContainer.append(buyAndSellWrapper);

        customBuyAndSellWithToggleWrapper.append(customBuyAndSellContainer);
        customBuyAndSellWithToggleWrapper.append(toggleBuyOrSellButton);
        buyAndSellButtonsContainer.append(customBuyAndSellWithToggleWrapper);

        insertElementBefore(chartsContainer, buyAndSellButtonsContainer);
      });
  } catch (error) {}
};

const injectNovaFlyingModal = () => {
  try {
    let showFlyingModal;
    let flyingModalPosition;
    let novaAuthToken;
    let elementsClassname;
    let buyButtonsList;
    let sellButtonsList;
    let activePresetLabel;
    let firstPresetValues;
    let secondPresetValues;
    let bsfm;
    chrome.storage.local
      .get([
        "show_flying_modal",
        "flying_modal_position",
        "nova_auth_token",
        "custom_buy_value_list",
        "custom_sell_value_list",
        "elements_classname",
        "active_preset_label",
        "first_preset_values",
        "second_preset_values",
        "bsfm",
      ])
      .then((result) => {
        showFlyingModal = result.show_flying_modal;
        flyingModalPosition = result.flying_modal_position;
        elementsClassname = result.elements_classname;
        novaAuthToken = result.nova_auth_token;
        activePresetLabel = result.active_preset_label;
        firstPresetValues = result.first_preset_values;
        secondPresetValues = result.second_preset_values;

        if (!result.bsfm) {
          const createdClassname = createRandomClassname();
          chrome.storage.local.set({
            bsfm: createdClassname,
          });
          bsfm = createdClassname;
        } else {
          bsfm = result.bsfm;
        }

        if (!showFlyingModal) return;

        if (!document.getElementById(elementsClassname?.i)) {
          const style = document.createElement("style");
          style.id = "custom-style";
          style.type = "text/css";
          style.textContent = `
            input[type="number"]::-webkit-inner-spin-button,
            input[type="number"]::-webkit-outer-spin-button {
              -webkit-appearance: none;
              margin: 0;
            }
            input[type="number"] {
              -moz-appearance: textfield; /* For Firefox */
              appearance: textfield; /* For modern browsers */
            }
          `;
          document.head.appendChild(style);
        }

        if (
          Array.isArray(result.custom_buy_value_list) &&
          result.custom_buy_value_list.length > 0
        ) {
          buyButtonsList = result.custom_buy_value_list;
        } else {
          buyButtonsList = [0.5, 1, 2, 5, 10];
        }
        if (
          Array.isArray(result.custom_sell_value_list) &&
          result.custom_sell_value_list.length > 0
        ) {
          sellButtonsList = result.custom_sell_value_list;
        } else {
          sellButtonsList = [10, 25, 50, 100];
        }

        const url = new URL(window.location.href);
        let tokenMintAddress = url.pathname.split("/meme/")[1];
        if (!tokenMintAddress) return;

        const previousDraggableNovaModal = document.querySelector(
          `div.${CSS.escape(bsfm)}`
        );
        if (previousDraggableNovaModal) {
          previousDraggableNovaModal.remove();
        }

        let offsetX = 0;
        let offsetY = 0;
        let isDragging = false;
        const draggable = document.createElement("div");
        draggable.classList.add(bsfm);
        Object.assign(draggable.style, {
          boxSizing: "border-box",
          position: "fixed",
          zIndex: "9999",
          width: "290px",
          height: "auto",
          paddingTop: "12px",
          backgroundColor: "rgba(13, 13, 16, 0.95)",
          top: `${flyingModalPosition.top}px`,
          left: `${flyingModalPosition.left}px`,
          display: "flex",
          flexDirection: "column",
          alignItems: "center",
          justifyContent: "center",
          userSelect: "none",
          border: "1px dashed rgba(255,83,236,0.7)",
          borderRadius: "6px",
          boxShadow:
            "0 0px 20px -1px rgba(169,0,150,0.64), inset 0 0px 20px -5px rgba(255,83,236,0.4)",
        });

        let isResizing = false;
        let baseScale = 1.0;
        let flyingModalSQ = null;
        let resizeOverlay = null;
        const createResizeElements = () => {
          const resizeContainer = document.createElement("div");
          Object.assign(resizeContainer.style, {
            position: "absolute",
            width: "13px",
            height: "13px",
            right: "0",
            bottom: "0",
            overflow: "hidden",
            zIndex: 10000,
          });

          const resizeElement = document.createElement("div");
          Object.assign(resizeElement.style, {
            position: "absolute",
            width: "9px",
            height: "9px",
            right: "2px",
            bottom: "2px",
            cursor: "nwse-resize",
            zIndex: 10000,
            display: "grid",
            gridTemplateColumns: "repeat(3, 1.5px)",
            gap: "1px",
            padding: "1px",
          });
          for (let i = 0; i < 9; i++) {
            const dot = document.createElement("div");
            Object.assign(dot.style, {
              width: "1.5px",
              height: "1.5px",
              backgroundColor: "rgba(255,83,236,0.7)",
              borderRadius: "0.75px",
            });
            resizeElement.appendChild(dot);
          }

          resizeContainer.appendChild(resizeElement);
          draggable.appendChild(resizeContainer);

          return resizeElement;
        };

        const showResizeOverlay = () => {
          resizeOverlay = document.createElement("div");
          Object.assign(resizeOverlay.style, {
            position: "fixed",
            top: 0,
            left: 0,
            width: "100vw",
            height: "100vh",
            zIndex: 999999,
            cursor: "nwse-resize",
            background: "transparent",
          });
          document.body.appendChild(resizeOverlay);
        };
        const hideResizeOverlay = () => {
          if (resizeOverlay && resizeOverlay.parentNode) {
            resizeOverlay.parentNode.removeChild(resizeOverlay);
            resizeOverlay = null;
          }
        };

        const getInitialScale = () => {
          return new Promise((resolve) => {
            chrome.storage.local.get("flying_modal_scale", (result) => {
              resolve(result.flying_modal_scale || { x: 1.0, y: 1.0 });
            });
          });
        };
        const applyScale = (scale) => {
          draggable.style.transform = `scale(${scale.x}, ${scale.y})`;
        };

        const onMouseDown = (event) => {
          event.preventDefault();
          event.stopPropagation();

          isResizing = true;
          flyingModalSQ = draggable.getBoundingClientRect();
          baseScale = 1.0;
          initialMouseX = event.clientX;
          initialMouseY = event.clientY;

          showResizeOverlay();
          document.addEventListener("mousemove", onMouseMove);
          document.addEventListener("mouseup", onMouseUp);
        };
        const onMouseMove = (event) => {
          if (!isResizing) return;

          const currentX = event.clientX;
          const currentY = event.clientY;

          const rawScaleX =
            (currentX - flyingModalSQ.left) / flyingModalSQ.width;
          const rawScaleY =
            (currentY - flyingModalSQ.top) / flyingModalSQ.height;
          const newScale = Math.min(rawScaleX, rawScaleY);

          const minScale = 0.85;
          const maxScale = 2.0;

          if (newScale < minScale || newScale > maxScale) return;

          draggable.style.transform = `scale(${newScale}, ${newScale})`;

          chrome.storage.local.set({
            flying_modal_scale: { x: newScale, y: newScale },
          });
        };
        const onMouseUp = () => {
          isResizing = false;
          hideResizeOverlay();
          document.removeEventListener("mousemove", onMouseMove);
          document.removeEventListener("mouseup", onMouseUp);
        };

        const initializeResize = async () => {
          const initialScale = await getInitialScale();
          applyScale(initialScale);

          const resizeElement = createResizeElements();
          resizeElement.addEventListener("mousedown", onMouseDown);
        };
        initializeResize();

        const header = document.createElement("div");
        Object.assign(header.style, {
          boxSizing: "border-box",
          width: "100%",
          padding: "0 12px 12px 12px",
          display: "flex",
          justifyContent: "space-between",
          alignItems: "center",
          color: "white",
          columnGap: "6px",
          borderBottom: "1px dashed rgba(255, 255, 255, 0.1)",
          cursor: "move",
        });
        const leftHeaderContainer = document.createElement("div");
        Object.assign(leftHeaderContainer.style, {
          width: "fit",
          display: "flex",
          alignItems: "center",
          color: "white",
          columnGap: "8px",
        });
        const headerText = document.createElement("h5");
        headerText.textContent = "Presets";
        Object.assign(headerText.style, {
          fontWeight: "700",
          fontSize: "16px",
          margin: "0px",
          color: "white",
        });
        let presets = [
          {
            label: "S1",
            active: activePresetLabel === "S1" ? true : false,
            values: firstPresetValues,
          },
          {
            label: "S2",
            active: activePresetLabel === "S2" ? true : false,
            values: secondPresetValues,
          },
        ];
        const presetsContainer = document.createElement("div");
        Object.assign(presetsContainer.style, {
          display: "flex",
          alignItems: "center",
          columnGap: "6px",
        });
        leftHeaderContainer.append(headerText, presetsContainer);
        const rightHeaderContainer = document.createElement("div");
        Object.assign(rightHeaderContainer.style, {
          width: "fit",
          display: "flex",
          alignItems: "center",
          color: "white",
          columnGap: "6px",
        });
        const settingSVG = cleanEscapedSvg(elementsClassname?.svgs?.setting);
        const backSVG = cleanEscapedSvg(elementsClassname?.svgs?.back);
        const settingButton = document.createElement("button");
        settingButton.innerHTML = settingSVG;
        Object.assign(settingButton.style, {
          boxSizing: "border-box",
          width: "20px",
          height: "20px",
          padding: 0,
          border: 0,
          background: "transparent",
          color: "white",
          cursor: "pointer",
        });
        settingButton.addEventListener("click", () => {
          if (settingContainer.style.display === "none") {
            settingButton.innerHTML = backSVG;
            buyAndSellContainer.style.display = "none";
            settingContainer.style.display = "flex";
          } else {
            settingButton.innerHTML = settingSVG;
            buyAndSellContainer.style.display = "flex";
            settingContainer.style.display = "none";
          }
        });

        const closeSVG = cleanEscapedSvg(elementsClassname?.svgs?.close);
        const closeButton = document.createElement("button");
        closeButton.innerHTML = closeSVG;
        Object.assign(closeButton.style, {
          boxSizing: "border-box",
          width: "24px",
          height: "24px",
          padding: 0,
          border: 0,
          background: "transparent",
          color: "white",
          cursor: "pointer",
          marginBottom: "1px",
        });
        closeButton.addEventListener("click", () => {
          const currentDraggableNovaModal = document.querySelector(
            `div.${CSS.escape(bsfm)}`
          );

          if (currentDraggableNovaModal) {
            currentDraggableNovaModal.remove();
            chrome.storage.local.set(
              {
                show_flying_modal: false,
                flying_modal_scale: { x: 1.0, y: 1.0 },
              },
              () => {}
            );
          }
        });

        rightHeaderContainer.append(settingButton, closeButton);
        header.append(leftHeaderContainer, rightHeaderContainer);

        header.addEventListener("mousedown", (e) => {
          isDragging = true;
          offsetX = e.clientX - draggable.offsetLeft;
          offsetY = e.clientY - draggable.offsetTop;
        });
        document.addEventListener("mousemove", (e) => {
          if (isDragging) {
            e.preventDefault();

            let newLeft = e.clientX - offsetX;
            let newTop = e.clientY - offsetY;

            const modalWidth = draggable.offsetWidth + 20;
            const modalHeight = draggable.offsetHeight + 20;

            const maxLeft = window.innerWidth - modalWidth;
            const maxTop = window.innerHeight - modalHeight;

            newLeft = Math.max(0, Math.min(newLeft, maxLeft));
            newTop = Math.max(0, Math.min(newTop, maxTop));

            draggable.style.left = `${newLeft + 10}px`;
            draggable.style.top = `${newTop + 10}px`;

            saveFlyingModalPosition(newLeft + 10, newTop + 10);
          }
        });
        document.addEventListener("mouseup", () => {
          isDragging = false;
        });

        const buyAndSellContainer = document.createElement("div");
        Object.assign(buyAndSellContainer.style, {
          boxSizing: "border-box",
          width: "100%",
          height: "auto",
          padding: "8px 12px 16px 12px",
          display: "flex",
          flexDirection: "column",
          rowGap: "12px",
          cursor: "default",
        });

        const buyContainer = document.createElement("div");
        Object.assign(buyContainer.style, {
          width: "100%",
          display: "flex",
          flexDirection: "column",
          rowGap: "4px",
        });
        const buyHeaderText = document.createElement("span");
        buyHeaderText.textContent = "Buy";
        Object.assign(buyHeaderText.style, {
          display: "block",
          fontWeight: "600",
          fontSize: "14px",
          margin: "0px",
          color: "rgba(255, 255, 255, 0.9)",
        });
        const buyGrid = document.createElement("div");
        Object.assign(buyGrid.style, {
          display: "grid",
          gridTemplateColumns: "repeat(5, 1fr)",
          gap: "6px",
        });
        buyGrid.innerHTML = "";

        const handleBuyButtonClick = async (buttonValue, customBuyButton) => {
          customBuyButton.disabled = true;
          customBuyButton.querySelector("span").textContent = "...";

          try {
            const storageData = await new Promise((resolve, reject) => {
              chrome.storage.local.get(
                ["default_buy_amount", "active_preset_values"],
                (result) => {
                  if (chrome.runtime.lastError) {
                    reject(chrome.runtime.lastError);
                  } else {
                    resolve(result);
                  }
                }
              );
            });

            const tokenInfo = await fetchTokenInfo(tokenMintAddress);
            const result = await transactToken(
              tokenInfo.tokenAddress,
              "buy",
              buttonValue,
              novaAuthToken,
              storageData?.active_preset_values
            );

            if (result) {
              customBuyButton.querySelector("span").textContent = "✅";
            } else {
              customBuyButton.querySelector("span").textContent = "❌";
            }
          } catch (error) {
            console.error("Error accessing Chrome storage:", error);
            customBuyButton.querySelector("span").textContent = "Error!";
          } finally {
            setTimeout(() => {
              customBuyButton.disabled = false;
              customBuyButton.querySelector("span").textContent =
                `${buttonValue}`;
            }, 700);
          }
        };
        buyButtonsList.forEach((value) => {
          const customBuyButton = document.createElement("button");
          customBuyButton.type = "button";

          Object.assign(customBuyButton.style, {
            width: "100%",
            padding: "5px 8px",
            display: "flex",
            justifyContent: "center",
            alignItems: "center",
            columnGap: "5px",
            border: "1px solid rgba(255,83,236,0.6)",
            borderRadius: "5px",
            background: "transparent",
            cursor: "pointer",
            color: "white",
            fontWeight: "600",
          });

          const buttonText = document.createElement("span");
          buttonText.style.color = "white";
          Object.assign(buttonText.style, {
            fontWeight: "600",
            color: "rgba(219,110,207,1)",
          });
          buttonText.textContent = value;

          customBuyButton.append(buttonText);

          customBuyButton.addEventListener("mouseenter", () => {
            customBuyButton.style.background = "rgba(255,83,236,0.1)";
          });

          customBuyButton.addEventListener("mouseleave", () => {
            customBuyButton.style.background = "transparent";
          });

          customBuyButton.onclick = () =>
            handleBuyButtonClick(value, customBuyButton);

          buyGrid.append(customBuyButton);
        });

        const buyCustomValueWrapper = document.createElement("div");
        Object.assign(buyCustomValueWrapper.style, {
          width: "100%",
          display: "flex",
          columnGap: "5px",
          marginTop: "5px",
        });
        const buyCustomValueInput = document.createElement("input");
        buyCustomValueInput.name = elementsClassname?.customs?.b;
        buyCustomValueInput.type = "number";
        buyCustomValueInput.placeholder =
          elementsClassname?.container?.placeholders?.buy?.amount;
        buyCustomValueInput.addEventListener("focus", () => {
          buyCustomValueInput.style.outline = "none";
        });
        buyCustomValueInput.addEventListener("input", (e) => {
          const value = e.target.value;
          const isValid = /^[1-9]\d*(\.\d+)?$|^0(\.\d+)?$/.test(value);
          if (!isValid) {
            e.target.value = value.slice(0, -1);
          }
        });
        Object.assign(buyCustomValueInput.style, {
          boxSizing: "border-box",
          width: "100%",
          padding: "5px 12px",
          columnGap: "5px",
          border: "1px solid rgba(255,83,236,0.6)",
          borderRadius: "5px",
          background: "transparent",
          color: "rgba(219,110,207,1)",
          placeholder: "rgba(219,110,207,1)",
        });
        const buyCustomValueButton = document.createElement("button");
        buyCustomValueButton.textContent = "Buy";
        Object.assign(buyCustomValueButton.style, {
          flexShrink: "0",
          width: "auto",
          padding: "6px 16px",
          columnGap: "5px",
          border: "1px solid rgba(255,83,236,0.6)",
          borderRadius: "5px",
          background: "rgba(169,0,150,0.64)",
          cursor: "pointer",
          color: "white",
          fontWeight: "bold",
        });

        buyCustomValueButton.addEventListener("click", async () => {
          const buyInputElement = document.querySelector(
            `[name=${elementsClassname?.customs?.b}]`
          );
          const buyValue = parseFloat(buyInputElement.value);
          buyCustomValueButton.disabled = true;
          buyCustomValueButton.textContent = "...";

          try {
            const storageData = await new Promise((resolve, reject) => {
              chrome.storage.local.get(
                ["default_buy_amount", "active_preset_values"],
                (result) => {
                  if (chrome.runtime.lastError) {
                    reject(chrome.runtime.lastError);
                  } else {
                    resolve(result);
                  }
                }
              );
            });

            const tokenInfo = await fetchTokenInfo(tokenMintAddress);
            const result = await transactToken(
              tokenInfo.tokenAddress,
              "buy",
              buyValue,
              novaAuthToken,
              storageData?.active_preset_values
            );

            if (result) {
              buyCustomValueButton.textContent = "Success!";
              setTimeout(() => {
                buyCustomValueButton.textContent = "Buy";
                buyCustomValueButton.disabled = false;
                buyInputElement.value = "";
              }, 700);
            } else {
              buyCustomValueButton.textContent = "Failed!";
              setTimeout(() => {
                buyCustomValueButton.textContent = "Buy";
                buyCustomValueButton.disabled = false;
                buyInputElement.value = "";
              }, 700);
            }
          } catch (error) {
            console.error("Error accessing Chrome storage:", error);
            buyCustomValueButton.textContent = "Error!";
            setTimeout(() => {
              buyCustomValueButton.textContent = "Buy";
              buyCustomValueButton.disabled = false;
              buyInputElement.value = "";
            }, 700);
          }
        });

        buyCustomValueWrapper.append(buyCustomValueInput, buyCustomValueButton);
        buyContainer.append(buyHeaderText, buyGrid, buyCustomValueWrapper);

        const sellContainer = document.createElement("div");
        Object.assign(sellContainer.style, {
          width: "100%",
          display: "flex",
          flexDirection: "column",
          rowGap: "4px",
        });
        const sellHeaderText = document.createElement("span");
        sellHeaderText.textContent = "Sell";
        Object.assign(sellHeaderText.style, {
          display: "block",
          fontWeight: "600",
          fontSize: "14px",
          margin: "0px",
          color: "rgba(255, 255, 255, 0.9)",
        });
        const sellGrid = document.createElement("div");
        Object.assign(sellGrid.style, {
          display: "grid",
          gridTemplateColumns: "repeat(5, 1fr)",
          gap: "6px",
        });

        const handleSellButtonClick = async (buttonValue, customSellButton) => {
          customSellButton.disabled = true;
          customSellButton.querySelector("span").textContent = "...";

          try {
            const storageData = await new Promise((resolve, reject) => {
              chrome.storage.local.get(
                ["default_buy_amount", "active_preset_values"],
                (result) => {
                  if (chrome.runtime.lastError) {
                    reject(chrome.runtime.lastError);
                  } else {
                    resolve(result);
                  }
                }
              );
            });

            const tokenInfo = await fetchTokenInfo(tokenMintAddress);
            const result = await transactToken(
              tokenInfo.tokenAddress,
              "sell",
              buttonValue,
              novaAuthToken,
              storageData?.active_preset_values
            );

            if (result) {
              customSellButton.querySelector("span").textContent = "✅";
            } else {
              customSellButton.querySelector("span").textContent = "❌";
            }
          } catch (error) {
            console.error("Error accessing Chrome storage:", error);
            customSellButton.querySelector("span").textContent = "Error!";
          } finally {
            setTimeout(() => {
              customSellButton.disabled = false;
              customSellButton.querySelector("span").textContent =
                `${buttonValue}%`;
            }, 700);
          }
        };
        sellButtonsList.forEach((value) => {
          const customSellButton = document.createElement("button");
          customSellButton.type = "button";

          Object.assign(customSellButton.style, {
            width: "100%",
            padding: "5px 8px",
            display: "flex",
            justifyContent: "center",
            alignItems: "center",
            columnGap: "5px",
            border: "1px solid rgba(180,180,180,0.6)",
            borderRadius: "5px",
            background: "transparent",
            cursor: "pointer",
            color: "white",
            fontWeight: "600",
            transition: "background 0.1s ease",
          });

          const buttonText = document.createElement("span");
          buttonText.style.color = "white";
          Object.assign(buttonText.style, {
            fontWeight: "600",
            color: "rgba(130,130,130,1)",
          });
          buttonText.textContent = `${value}%`;

          customSellButton.append(buttonText);

          customSellButton.addEventListener("mouseenter", () => {
            customSellButton.style.background = "rgba(255,255,255, 0.08)";
            buttonText.style.color = "rgba(160,160,160,1)";
          });

          customSellButton.addEventListener("mouseleave", () => {
            customSellButton.style.background = "transparent";
            buttonText.style.color = "rgba(130,130,130,1)";
          });

          customSellButton.onclick = () =>
            handleSellButtonClick(value, customSellButton);

          sellGrid.append(customSellButton);
        });

        const sellCustomValueWrapper = document.createElement("div");
        Object.assign(sellCustomValueWrapper.style, {
          width: "100%",
          display: "flex",
          columnGap: "5px",
          marginTop: "5px",
        });
        const sellCustomValueInput = document.createElement("input");
        sellCustomValueInput.name = elementsClassname?.customs?.s;
        sellCustomValueInput.type = "number";
        sellCustomValueInput.placeholder =
          elementsClassname?.container?.placeholders?.sell?.percentage;
        sellCustomValueInput.addEventListener("focus", () => {
          sellCustomValueInput.style.outline = "none";
        });
        sellCustomValueInput.addEventListener("input", (e) => {
          const value = e.target.value;
          const isValid = /^[1-9]$|^[1-9][0-9]$|^100$/.test(value);

          if (!isValid) {
            e.target.value = value.slice(0, -1);
          }
        });
        sellCustomValueInput.addEventListener("keydown", (e) => {
          if (e.key === ".") {
            e.preventDefault();
          }
        });
        Object.assign(sellCustomValueInput.style, {
          boxSizing: "border-box",
          width: "100%",
          padding: "5px 12px",
          columnGap: "5px",
          border: "1px solid rgba(255,83,236,0.6)",
          borderRadius: "5px",
          background: "transparent",
          color: "rgba(219,110,207,1)",
          placeholder: "rgba(219,110,207,1)",
        });
        const sellCustomValueButton = document.createElement("button");
        sellCustomValueButton.textContent = "Sell";
        Object.assign(sellCustomValueButton.style, {
          flexShrink: "0",
          width: "auto",
          padding: "6px 16px",
          columnGap: "5px",
          border: "1px solid rgba(255,83,236,0.6)",
          borderRadius: "5px",
          background: "rgba(169,0,150,0.64)",
          cursor: "pointer",
          color: "white",
          fontWeight: "bold",
        });

        sellCustomValueButton.addEventListener("click", async () => {
          const sellInputElement = document.querySelector(
            `[name=${elementsClassname?.customs?.s}]`
          );
          const sellValue = parseFloat(sellInputElement.value);
          sellCustomValueButton.disabled = true;
          sellCustomValueButton.textContent = "...";

          try {
            const storageData = await new Promise((resolve, reject) => {
              chrome.storage.local.get(
                ["default_buy_amount", "active_preset_values"],
                (result) => {
                  if (chrome.runtime.lastError) {
                    reject(chrome.runtime.lastError);
                  } else {
                    resolve(result);
                  }
                }
              );
            });

            const tokenInfo = await fetchTokenInfo(tokenMintAddress);
            const result = await transactToken(
              tokenInfo.tokenAddress,
              "sell",
              sellValue,
              novaAuthToken,
              storageData?.active_preset_values
            );

            if (result) {
              sellCustomValueButton.textContent = "Success!";
              setTimeout(() => {
                sellCustomValueButton.textContent = "Sell";
                sellCustomValueButton.disabled = false;
                sellInputElement.value = "";
              }, 700);
            } else {
              sellCustomValueButton.textContent = "Failed!";
              setTimeout(() => {
                sellCustomValueButton.textContent = "Sell";
                sellCustomValueButton.disabled = false;
                sellInputElement.value = "";
              }, 700);
            }
          } catch (error) {
            console.error("Error accessing Chrome storage:", error);
            sellCustomValueButton.textContent = "Error!";
            setTimeout(() => {
              sellCustomValueButton.textContent = "Sell";
              sellCustomValueButton.disabled = false;
              sellInputElement.value = "";
            }, 700);
          }
        });

        sellCustomValueWrapper.append(
          sellCustomValueInput,
          sellCustomValueButton
        );
        sellContainer.append(sellHeaderText, sellGrid, sellCustomValueWrapper);

        const settingContainer = document.createElement("div");
        Object.assign(settingContainer.style, {
          boxSizing: "border-box",
          width: "100%",
          height: "auto",
          padding: "8px 12px 16px 12px",
          display: "none",
          flexDirection: "column",
          rowGap: "12px",
          cursor: "default",
        });
        const settingHeaderText = document.createElement("span");
        settingHeaderText.textContent = "Settings";
        Object.assign(settingHeaderText.style, {
          display: "block",
          fontWeight: "600",
          fontSize: "14px",
          margin: "0px",
          color: "rgba(255, 255, 255, 0.9)",
        });
        const settingGrid = document.createElement("div");
        Object.assign(settingGrid.style, {
          width: "100%",
          display: "grid",
          gridTemplateColumns: "repeat(3, 1fr)",
          gap: "6px",
        });
        const buyFeeWrapper = document.createElement("div");
        const buyFeeHeader = document.createElement("span");
        buyFeeHeader.textContent = "Buy Fee";
        Object.assign(buyFeeHeader.style, {
          display: "block",
          fontSize: "10px",
          marginBottom: "3.5px",
          color: "rgba(255, 255, 255, 0.6)",
        });
        const buyFeeInput = document.createElement("input");
        buyFeeInput.name = elementsClassname?.settings["buy-fee"];
        buyFeeInput.type = "number";
        buyFeeInput.placeholder = "(SOL)";
        buyFeeInput.addEventListener("focus", () => {
          buyFeeInput.style.outline = "none";
        });
        buyFeeInput.addEventListener("input", (e) => {
          const value = e.target.value;
          const isValid = /^[1-9]\d*(\.\d+)?$|^0(\.\d+)?$/.test(value);
          if (!isValid) {
            e.target.value = value.slice(0, -1);
          }
        });
        Object.assign(buyFeeInput.style, {
          boxSizing: "border-box",
          width: "100%",
          padding: "5px 12px",
          columnGap: "5px",
          border: "1px solid rgba(255,83,236,0.6)",
          borderRadius: "5px",
          background: "transparent",
          color: "rgba(219,110,207,1)",
          placeholder: "rgba(219,110,207,1)",
          textAlign: "center",
        });
        buyFeeWrapper.append(buyFeeHeader, buyFeeInput);
        const buyTipWrapper = document.createElement("div");
        const buyTipHeader = document.createElement("span");
        buyTipHeader.textContent = "Buy Tip";
        Object.assign(buyTipHeader.style, {
          display: "block",
          fontSize: "10px",
          marginBottom: "3.5px",
          color: "rgba(255, 255, 255, 0.6)",
        });
        const buyTipInput = document.createElement("input");
        buyTipInput.name = elementsClassname?.settings["buy-tip"];
        buyTipInput.type = "number";
        buyTipInput.placeholder = "(SOL)";
        buyTipInput.addEventListener("focus", () => {
          buyTipInput.style.outline = "none";
        });
        buyTipInput.addEventListener("input", (e) => {
          const value = e.target.value;
          const isValid = /^[1-9]\d*(\.\d+)?$|^0(\.\d+)?$/.test(value);
          if (!isValid) {
            e.target.value = value.slice(0, -1);
          }
        });
        Object.assign(buyTipInput.style, {
          boxSizing: "border-box",
          width: "100%",
          padding: "5px 12px",
          columnGap: "5px",
          border: "1px solid rgba(255,83,236,0.6)",
          borderRadius: "5px",
          background: "transparent",
          color: "rgba(219,110,207,1)",
          placeholder: "rgba(219,110,207,1)",
          textAlign: "center",
        });
        buyTipWrapper.append(buyTipHeader, buyTipInput);
        const buySlippageWrapper = document.createElement("div");
        const buySlippageHeader = document.createElement("span");
        buySlippageHeader.textContent = "Buy Slippage";
        Object.assign(buySlippageHeader.style, {
          display: "block",
          fontSize: "10px",
          marginBottom: "3.5px",
          color: "rgba(255, 255, 255, 0.6)",
        });
        const buySlippageInput = document.createElement("input");
        buySlippageInput.name = elementsClassname?.settings["buy-slippage"];
        buySlippageInput.type = "number";
        buySlippageInput.placeholder = "(%)";
        buySlippageInput.addEventListener("focus", () => {
          buySlippageInput.style.outline = "none";
        });
        buySlippageInput.addEventListener("input", (e) => {
          const value = e.target.value;
          const isValid = /^[1-9]\d*(\.\d+)?$|^0(\.\d+)?$/.test(value);
          if (!isValid) {
            e.target.value = value.slice(0, -1);
          }
        });
        Object.assign(buySlippageInput.style, {
          boxSizing: "border-box",
          width: "100%",
          padding: "5px 12px",
          columnGap: "5px",
          border: "1px solid rgba(255,83,236,0.6)",
          borderRadius: "5px",
          background: "transparent",
          color: "rgba(219,110,207,1)",
          placeholder: "rgba(219,110,207,1)",
          textAlign: "center",
        });
        buySlippageWrapper.append(buySlippageHeader, buySlippageInput);
        const sellFeeWrapper = document.createElement("div");
        const sellFeeHeader = document.createElement("span");
        sellFeeHeader.textContent = "Sell Fee";
        Object.assign(sellFeeHeader.style, {
          display: "block",
          fontSize: "10px",
          marginBottom: "3.5px",
          color: "rgba(255, 255, 255, 0.6)",
        });
        const sellFeeInput = document.createElement("input");
        sellFeeInput.name = elementsClassname?.settings["sell-fee"];
        sellFeeInput.type = "number";
        sellFeeInput.placeholder = "(SOL)";
        sellFeeInput.addEventListener("focus", () => {
          sellFeeInput.style.outline = "none";
        });
        sellFeeInput.addEventListener("input", (e) => {
          const value = e.target.value;
          const isValid = /^[1-9]\d*(\.\d+)?$|^0(\.\d+)?$/.test(value);
          if (!isValid) {
            e.target.value = value.slice(0, -1);
          }
        });
        Object.assign(sellFeeInput.style, {
          boxSizing: "border-box",
          width: "100%",
          padding: "5px 12px",
          columnGap: "5px",
          border: "1px solid rgba(255,83,236,0.6)",
          borderRadius: "5px",
          background: "transparent",
          color: "rgba(219,110,207,1)",
          placeholder: "rgba(219,110,207,1)",
          textAlign: "center",
        });
        sellFeeWrapper.append(sellFeeHeader, sellFeeInput);
        const sellTipWrapper = document.createElement("div");
        const sellTipHeader = document.createElement("span");
        sellTipHeader.textContent = "Sell Tip";
        Object.assign(sellTipHeader.style, {
          display: "block",
          fontSize: "10px",
          marginBottom: "3.5px",
          color: "rgba(255, 255, 255, 0.6)",
        });
        const sellTipInput = document.createElement("input");
        sellTipInput.name = elementsClassname?.settings["sell-tip"];
        sellTipInput.type = "number";
        sellTipInput.placeholder = "(SOL)";
        sellTipInput.addEventListener("focus", () => {
          sellTipInput.style.outline = "none";
        });
        sellTipInput.addEventListener("input", (e) => {
          const value = e.target.value;
          const isValid = /^[1-9]\d*(\.\d+)?$|^0(\.\d+)?$/.test(value);
          if (!isValid) {
            e.target.value = value.slice(0, -1);
          }
        });
        Object.assign(sellTipInput.style, {
          boxSizing: "border-box",
          width: "100%",
          padding: "5px 12px",
          columnGap: "5px",
          border: "1px solid rgba(255,83,236,0.6)",
          borderRadius: "5px",
          background: "transparent",
          color: "rgba(219,110,207,1)",
          placeholder: "rgba(219,110,207,1)",
          textAlign: "center",
        });
        sellTipWrapper.append(sellTipHeader, sellTipInput);
        const sellSlippageWrapper = document.createElement("div");
        const sellSlippageHeader = document.createElement("span");
        sellSlippageHeader.textContent = "Sell Slippage";
        Object.assign(sellSlippageHeader.style, {
          display: "block",
          fontSize: "10px",
          marginBottom: "3.5px",
          color: "rgba(255, 255, 255, 0.6)",
        });
        const sellSlippageInput = document.createElement("input");
        sellSlippageInput.name = elementsClassname?.settings["sell-slippage"];
        sellSlippageInput.type = "number";
        sellSlippageInput.placeholder = "(%)";
        sellSlippageInput.addEventListener("focus", () => {
          sellSlippageInput.style.outline = "none";
        });
        sellSlippageInput.addEventListener("input", (e) => {
          const value = e.target.value;
          const isValid = /^[1-9]\d*(\.\d+)?$|^0(\.\d+)?$/.test(value);
          if (!isValid) {
            e.target.value = value.slice(0, -1);
          }
        });
        Object.assign(sellSlippageInput.style, {
          boxSizing: "border-box",
          width: "100%",
          padding: "5px 12px",
          columnGap: "5px",
          border: "1px solid rgba(255,83,236,0.6)",
          borderRadius: "5px",
          background: "transparent",
          color: "rgba(219,110,207,1)",
          placeholder: "rgba(219,110,207,1)",
          textAlign: "center",
        });
        sellSlippageWrapper.append(sellSlippageHeader, sellSlippageInput);
        const saveSettingButton = document.createElement("button");
        saveSettingButton.textContent = "Save Settings";
        Object.assign(saveSettingButton.style, {
          flexShrink: "0",
          width: "auto",
          padding: "6px 16px",
          columnGap: "5px",
          border: "1px solid rgba(255,83,236,0.6)",
          borderRadius: "5px",
          background: "rgba(169,0,150,0.64)",
          cursor: "pointer",
          color: "white",
          fontWeight: "bold",
        });
        saveSettingButton.addEventListener("click", () => {
          const activePreset = presets.find((preset) => preset.active);

          if (activePreset) {
            activePreset.values["buy-fee"] =
              parseFloat(buyFeeInput.value) || 0.001;
            activePreset.values["buy-tip"] =
              parseFloat(buyTipInput.value) || 0.005;
            activePreset.values["buy-slippage"] =
              parseFloat(buySlippageInput.value) || 50;
            activePreset.values["sell-fee"] =
              parseFloat(sellFeeInput.value) || 0.001;
            activePreset.values["sell-tip"] =
              parseFloat(sellTipInput.value) || 0.005;
            activePreset.values["sell-slippage"] =
              parseFloat(sellSlippageInput.value) || 50;

            if (activePreset.label === "S1") {
              chrome.storage.local.set(
                {
                  first_preset_values: activePreset?.values,
                  active_preset_values: activePreset.values,
                },
                () => {}
              );
            } else {
              chrome.storage.local.set(
                {
                  second_preset_values: activePreset?.values,
                  active_preset_values: activePreset.values,
                },
                () => {}
              );
            }

            saveSettingButton.textContent = "Success!";
            setTimeout(() => {
              saveSettingButton.textContent = "Save Settings";
            }, 1000);
          } else {
          }
        });
        const renderPresets = () => {
          presetsContainer.innerHTML = "";

          presets.forEach((preset) => {
            const presetButton = document.createElement("button");
            presetButton.textContent = preset?.label;
            Object.assign(presetButton.style, {
              flexShrink: "0",
              width: "auto",
              padding: "4px 12px",
              columnGap: "5px",
              border: "1px solid rgba(255,83,236,0.6)",
              borderRadius: "6px",
              background: preset?.active
                ? "rgba(169,0,150,0.64)"
                : "transparent",
              cursor: "pointer",
              color: "white",
              fontWeight: "bold",
            });

            if (!preset?.active) {
              presetButton.addEventListener("mouseenter", () => {
                presetButton.style.background = "rgba(255,83,236,0.1)";
              });

              presetButton.addEventListener("mouseleave", () => {
                presetButton.style.background = "transparent";
              });
            } else {
              buyFeeInput.value = preset?.values["buy-fee"];
              buyTipInput.value = preset?.values["buy-tip"];
              buySlippageInput.value = preset?.values["buy-slippage"];
              sellFeeInput.value = preset?.values["sell-fee"];
              sellTipInput.value = preset?.values["sell-tip"];
              sellSlippageInput.value = preset?.values["sell-slippage"];

              chrome.storage.local.set(
                {
                  active_preset_values: preset?.values,
                },
                () => {}
              );
            }

            presetButton.addEventListener("click", () => {
              chrome.storage.local.set(
                {
                  active_preset_label: preset?.label,
                },
                () => {}
              );

              presets.forEach((p) => {
                p.active = false;
              });

              preset.active = true;

              buyFeeInput.value = preset?.values["buy-fee"];
              buyTipInput.value = preset?.values["buy-tip"];
              buySlippageInput.value = preset?.values["buy-slippage"];
              sellFeeInput.value = preset?.values["sell-fee"];
              sellTipInput.value = preset?.values["sell-tip"];
              sellSlippageInput.value = preset?.values["sell-slippage"];
              renderPresets();
            });

            presetsContainer.append(presetButton);
          });
        };
        renderPresets();
        settingGrid.append(
          buyFeeWrapper,
          buyTipWrapper,
          buySlippageWrapper,
          sellFeeWrapper,
          sellTipWrapper,
          sellSlippageWrapper
        );
        settingContainer.append(
          settingHeaderText,
          settingGrid,
          saveSettingButton
        );

        draggable.append(header, buyAndSellContainer, settingContainer);
        buyAndSellContainer.append(buyContainer, sellContainer);

        document.querySelector("body").append(draggable);
      });
  } catch (error) {}
};

/**********************************************************
 * MEMESCOPE BUTTON INJECTION FUNCTIONS
 **********************************************************/

const buyButtonClassName = "b" + Math.random().toString(36).substring(2, 10);
const snipeButtonClassName = "n" + Math.random().toString(36).substring(2, 10);

const getTokenMintFromCard = (card) => {
  const tokenButton = card.querySelector("button[class*='group/copy']");
  if (tokenButton) {
    const tokenSpan = tokenButton.querySelector("span");
    return tokenSpan ? tokenSpan.textContent.trim() : null;
  }
  return null;
};

const getFullTokenAddress = (truncatedToken) => {
  if (!truncatedToken || !truncatedToken.includes("...")) return null;
  const [start, end] = truncatedToken.split("...");
  const pulseRaw = localStorage.getItem("axiom.pulse");
  if (!pulseRaw) return null;
  try {
    const pulseData = JSON.parse(pulseRaw);
    const token = pulseData.content.find(
      (t) => t.tokenAddress.startsWith(start) && t.tokenAddress.endsWith(end)
    );
    return token ? token.tokenAddress : null;
  } catch (e) {
    return null;
  }
};


const getInjectionContainer = (actionArea) => {
  return actionArea.parentElement ? actionArea.parentElement : actionArea;
};

const addNovaBuyButton = (card, actionArea) => {
  const buttonText = "Buy";
  const button = document.createElement("button");
  button.classList.add(buyButtonClassName);

 
  const logoImg = document.createElement("img");
  logoImg.src = chrome.runtime.getURL("images/logo.png");
  logoImg.alt = "Logo";
  logoImg.style.height = "16px";
  logoImg.style.marginRight = "4px";

  
  const textSpan = document.createElement("span");
  textSpan.textContent = buttonText;

 
  button.appendChild(logoImg);
  button.appendChild(textSpan);

  
  const originalContent = button.innerHTML;

 
  button.style.cssText =
    "display: inline-flex; align-items: center; flex-direction: row; border: 1px solid #EEA7ED; margin-left: 6px; padding: 2px 30px; border-radius: 4px; z-index: 1000; background-color: #800080; cursor: pointer; font-size: 14px; line-height: 16px;";

  
  button.addEventListener("mouseenter", () => {
    button.style.backgroundColor = "#9932CC";
  });
  button.addEventListener("mouseleave", () => {
    button.style.backgroundColor = "#800080";
  });

  button.onclick = async (event) => {
    event.preventDefault();
    event.stopPropagation();
    button.disabled = true;
    
    button.setAttribute("data-processing", "true");
    button.innerHTML = "...";
    const truncatedToken = getTokenMintFromCard(card);
   
    if (!truncatedToken) {
      console.warn("Truncated token not found");
      button.disabled = false;
      button.innerHTML = originalContent;
      button.removeAttribute("data-processing");
      return;
    }
    const fullTokenAddress = getFullTokenAddress(truncatedToken);
    if (!fullTokenAddress) {
      console.warn("Full token address not found");
      button.innerHTML = "❌";
      setTimeout(() => {
        button.innerHTML = buttonText;
        button.disabled = false;
        button.removeAttribute("data-processing");
      }, 700);
      return;
    }

    try {
      const { nova_auth_token, active_preset_values, default_buy_amount } =
        await chrome.storage.local.get([
          "nova_auth_token",
          "active_preset_values",
          "default_buy_amount",
        ]);
      const buyAmount = default_buy_amount || 0.01;
      const result = await transactToken(
        fullTokenAddress,
        "buy",
        buyAmount,
        nova_auth_token,
        active_preset_values
      );
      if (result) {
        button.innerHTML = "✅";
        setTimeout(() => {
          button.innerHTML = originalContent;
          button.disabled = false;
          button.removeAttribute("data-processing");
        }, 2000);
      } else {
        button.innerHTML = "❌";
        setTimeout(() => {
          button.innerHTML = originalContent;
          button.disabled = false;
          button.removeAttribute("data-processing");
        }, 2000);
      }
    } catch (error) {
      console.error("transactToken error:", error);
      button.innerHTML = "❌";
      setTimeout(() => {
        button.innerHTML = originalContent;
        button.disabled = false;
        button.removeAttribute("data-processing");
      }, 2000);
    }
  };

  
  const container = getInjectionContainer(actionArea);
  container.style.display = "flex";
  container.appendChild(button);
};

const addNovaSnipeButton = (card, actionArea) => {
  const buttonText = "Snipe";
  const button = document.createElement("button");
  button.classList.add(snipeButtonClassName);
  button.textContent = buttonText;
  
  button.style.cssText =
    "border: 1px solid #EEA7ED; margin-left: 6px; padding: 2px 20px; border-radius: 4px; z-index: 1000; background-color: #800080; cursor: pointer; font-size: 14px; line-height: 16px;";
  button.addEventListener("mouseenter", () => {
    button.style.backgroundColor = "#9932CC";
  });
  button.addEventListener("mouseleave", () => {
    button.style.backgroundColor = "#800080";
  });
  button.onclick = async (event) => {
    event.preventDefault();
    event.stopPropagation();
    button.disabled = true;
    button.setAttribute("data-processing", "true");
    button.textContent = "...";
    const truncatedToken = getTokenMintFromCard(card);
 
    if (!truncatedToken) {
      console.warn("Truncated token not found");
      button.disabled = false;
      button.removeAttribute("data-processing");
      return;
    }
    const fullTokenAddress = getFullTokenAddress(truncatedToken);
    if (!fullTokenAddress) {
      console.warn("Full token address not found");
      button.textContent = "❌";
      setTimeout(() => {
        button.textContent = buttonText;
        button.disabled = false;
        button.removeAttribute("data-processing");
      }, 2000);
      return;
    }

    try {
      const { nova_auth_token } = await chrome.storage.local.get(["nova_auth_token"]);
      const snipeAmount = 0; 
      const result = await transactToken(
        fullTokenAddress,
        "snipe",
        snipeAmount,
        nova_auth_token,
        {}
      );
      if (result) {
        button.textContent = "✅";
        setTimeout(() => {
          button.textContent = buttonText;
          button.disabled = false;
          button.removeAttribute("data-processing");
        }, 700);
      } else {
        button.textContent = "❌";
        setTimeout(() => {
          button.textContent = buttonText;
          button.disabled = false;
          button.removeAttribute("data-processing");
        }, 700);
      }
    } catch (error) {
      console.error("transactToken error:", error);
      button.textContent = "❌";
      setTimeout(() => {
        button.textContent = buttonText;
        button.disabled = false;
        button.removeAttribute("data-processing");
      }, 700);
    }
  };

  const container = getInjectionContainer(actionArea);
  container.appendChild(button);
};

const addNovaButtons = (card) => {
  try {
    const pumpLogo = card.querySelector(
      'img[src="https://axiom.trade/images/pump.svg"]'
    );
    const isMigrating = pumpLogo && pumpLogo.closest("div.bg-primaryBlue");

    let actionAreas = [];
    if (isMigrating) {
      actionAreas = Array.from(
        card.querySelectorAll("img[src*='pump-grad.svg']")
      ).map((el) => el.closest("div.relative"));
    } else {
      actionAreas = Array.from(
        card.querySelectorAll("button i.ri-flashlight-fill")
      ).map((el) => el.parentElement);
    }

    if (!actionAreas.length) {
      const tokenAnchor = card.querySelector('button[class*="group/copy"]');
      if (tokenAnchor) {
        actionAreas = [tokenAnchor.parentElement];
      }
    }
    if (!actionAreas.length) return;

    actionAreas.forEach((actionArea) => {
      
      const container = getInjectionContainer(actionArea);
      if (
        !container.querySelector(
          `button.${buyButtonClassName}, button.${snipeButtonClassName}`
        )
      ) {
        if (isMigrating) {
          addNovaSnipeButton(card, actionArea);
        } else {
          addNovaBuyButton(card, actionArea);
        }
      }
    });
  } catch (error) {
    console.error("Failed to add Nova button:", error);
  }
};

const injectNovaMemescopeButtons = async () => {
  try {
    const cards = Array.from(
      document.querySelectorAll("div[style*='position: absolute'][style*='width: 100%']")
    );
    cards.forEach((card) => {
      addNovaButtons(card);
    });
  } catch (error) {
    console.error("injectNovaMemescopeButtons error:", error);
  }
};

chrome.storage.local.get("is_nova_extension_on", (result) => {
  if (result.is_nova_extension_on) {
    let injectionTimeout;
    const memescopeObserver = new MutationObserver(() => {
      if (injectionTimeout) clearTimeout(injectionTimeout);
      injectionTimeout = setTimeout(() => {
        if (
          document.querySelector("div[style*='position: absolute'][style*='width: 100%']")
        ) {
          injectNovaMemescopeButtons();
        }
      }, 100);
    });
    memescopeObserver.observe(document.body, {
      childList: true,
      subtree: true,
    });

 
    if (document.readyState === "loading") {
      document.addEventListener("DOMContentLoaded", injectNovaMemescopeButtons);
    } else {
      injectNovaMemescopeButtons();
    }
  }
});

/**********************************************************
 * CHROME RUNTIME MESSAGE LISTENER
 **********************************************************/
chrome.runtime.onMessage.addListener(async function (request) {
  chrome.storage.local.get(
    ["is_nova_extension_on", "elements_classname", "bsfm", "srcn"],
    async (result) => {
      const isExtensionOn = result.is_nova_extension_on;
      const elementsClassname = result.elements_classname;
      const bsfm = result.bsfm;
      const srcn = result?.srcn;

      if (!isExtensionOn) return;

      if (request.message === "axiom-memescope") {
        const previousDraggableNovaModal = document.querySelector(
          `div.${CSS.escape(bsfm)}`
        );
        if (previousDraggableNovaModal) previousDraggableNovaModal.remove();

        const container = await searchAndFindTokenContainer(
          elementsClassname?.placement_selectors?.axiom?.memescope?.container
        );
        if (container) {
          injectNovaMemescopeButtons();
          const observer = new MutationObserver((mutations) => {
            mutations.forEach((mutation) => {
              Array.from(mutation.addedNodes).forEach((node) => {
                if (
                  node.nodeName === "SPAN" &&
                  node.classList.contains("contents")
                ) {
                  injectNovaMemescopeButtons();
                }
              });
            });
          });
          observer.observe(container, { childList: true, subtree: true });
        }
      }

      if (request.message === "axiom-token-save") {
        const currentUrl = window.location.href;
        if (currentUrl.includes("/meme")) {
          injectNovaFlyingModal();
          injectNovaContainer();
        }
      }

      if (request.message === "axiom-token") {
        injectNovaFlyingModal();
        const buySellContainer = await searchAndFindBuyAndSellContainer(
          elementsClassname?.placement_selectors?.axiom?.token?.snipe?.container
        );
        let runCount = 0;
        if (buySellContainer) {
          injectNovaSnipeButton();
          const observer = new MutationObserver(() => {
            const migrating = document.querySelectorAll(
              elementsClassname?.placement_selectors?.axiom?.token?.snipe
                ?.migrating
            )[1];
            if (migrating && runCount === 0) {
              runCount++;
              injectNovaSnipeButton();
            } else if (
              document.querySelector(
                elementsClassname?.placement_selectors?.axiom?.token?.snipe
                  ?.paragraph
              ) === null &&
              runCount > 0
            ) {
              document
                .querySelectorAll(`button.${CSS.escape(srcn)}`)
                .forEach((el) => el.remove());
            }
          });
          observer.observe(buySellContainer, {
            childList: true,
            subtree: true,
          });
        }

        const chartsContainer = await searchAndFindChartsContainer(
          elementsClassname?.placement_selectors?.axiom?.token?.container
        );
        if (chartsContainer) {
          injectNovaContainer();
        }
      }
    }
  );
});
