(function () {
  "use strict";

  const originalWebSocket = window.WebSocket;
  window.WebSocket = function WebSocket(url, protocols) {
    let ws;
    if (arguments.length === 1) {
      ws = new originalWebSocket(url);
    } else if (arguments.length >= 2) {
      ws = new originalWebSocket(url, protocols);
    } else {
      ws = new originalWebSocket();
    }

    ws.addEventListener("message", function (event) {
      try {
        if (event.data.toLowerCase().includes("update_pulse")) {
          let previousPulse = localStorage.getItem("axiom.pulse");
          try {
            previousPulse = previousPulse
              ? JSON.parse(previousPulse)
              : { content: [] };
          } catch (e) {
            previousPulse = { content: [] };
          }
          const newPulseData = JSON.parse(event.data);
          const cutoff = Date.now() - 300000;

          const filteredPrevious = previousPulse.content.filter(
            (token) => new Date(token.lastSeen).getTime() > cutoff
          );

          const mergedContents = [
            ...filteredPrevious.filter(
              (token) =>
                !newPulseData.content.some(
                  (newToken) => newToken.tokenAddress === token.tokenAddress
                )
            ),
            ...newPulseData.content.map((token) => ({
              ...token,
              lastSeen: new Date().toISOString(),
            })),
          ];

          localStorage.setItem(
            "axiom.pulse",
            JSON.stringify({ content: mergedContents })
          );
        }
      } catch (e) {
        console.error("WS message parse error:", e);
      }
    });

    return ws;
  };
  window.WebSocket.prototype = originalWebSocket.prototype;
  window.WebSocket.prototype.constructor = window.WebSocket;
})();
